from typing import Optional
import strawberry

from fast_mu_builder.attach.response import AttachmentResponse

'''ATTACHMENT_RESPONSE'''
@strawberry.type
class _MODEL_AttachmentResponse(AttachmentResponse):
    pass