import uuid
from datetime import datetime
from tortoise import models, fields

class TimeStampedModel(models.Model):
    id = fields.UUIDField(pk=True, default=uuid.uuid4)
    created_at = fields.DatetimeField(auto_now_add=True)
    updated_at = fields.DatetimeField(auto_now=True)

    class Meta:
        abstract = True  # Ensure no table is created for this model